Imports System.Xml
Imports System.IO

Public Class Form1
    Public Function ConvertDataSetToXML(ByVal ds As DataSet) As String

        Dim stream As MemoryStream = Nothing
        Dim writer As XmlTextWriter = Nothing
        Try
            stream = New MemoryStream()
            ' XmlTextWriter mit dem MemoryStream initialiseren:
            writer = New XmlTextWriter(stream, System.Text.Encoding.Unicode)
            ' DataSet in den MemoryStream schreiben und dabei auch die Strukturinformationen mit bergeben:
            ds.WriteXml(writer, XmlWriteMode.WriteSchema)
            ' Byte-Array als Puffer erstellen (MemoryStream kann grundstzlich nur in ein Byte-Array einlesen):
            Dim arr() As Byte = stream.ToArray()
            ' Xml-String aus Byte-Array gewinnen und zurckgeben:
            Dim utf As New System.Text.UnicodeEncoding()
            Return utf.GetString(arr).Trim()
        Catch
            Return String.Empty
        Finally
            If writer IsNot Nothing Then writer.Close()
        End Try
    End Function

    ' Die zweite Methode arbeitet in umgekehrter Richtung, sie konvertiert einen bergebenen Xml-String in ein DataSet, was dank StringReader-Objekt auf direktem Weg geht:
    Public Function ConvertXMLToDataSet(ByVal xml As String) As DataSet
        Dim reader As StringReader = Nothing
        Try
            Dim ds As New DataSet()
            reader = New StringReader(xml)
            ' Xml-String in DataSet einlesen: 
            ds.ReadXml(reader)
            Return ds
        Catch
            Return Nothing
        Finally
            If reader IsNot Nothing Then reader.Close()
        End Try
    End Function

    ' DataSet => Xml:
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Dim ds As DataSet = getTestDS()
        TextBox1.Text = ConvertDataSetToXML(ds)
        Button2.Enabled = True
    End Sub

    ' xml => DataSet:
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Dim ds As DataSet = ConvertXMLToDataSet(TextBox1.Text)
        DataGrid1.DataSource = Nothing
        DataGrid1.DataSource = ds
    End Sub


    Private Function getTestDS() As DataSet

        'Tabelle Personen:
        Dim dt1 As New DataTable("Personen")
        'Primrschlssel:
        Dim col1 As DataColumn = dt1.Columns.Add("Nr", GetType(System.Int32))
        col1.AllowDBNull = False
        col1.Unique = True
        col1.AutoIncrement = True
        col1.AutoIncrementStep = 1

        ' restliche Spalten hinzufgen:
        dt1.Columns.Add("Vorname", GetType(System.String))
        dt1.Columns.Add("Nachname", GetType(System.String))
        dt1.Columns.Add("Geburtstag", GetType(System.DateTime))

        ' zwei Datenstze hinzufgen:
        Dim rw11 As DataRow = dt1.NewRow()
        rw11("Vorname") = "Klaus"
        rw11("Nachname") = "Mller"
        rw11("Geburtstag") = Convert.ToDateTime("3.4.1975")
        Dim rw12 As DataRow = dt1.NewRow()
        rw12("Vorname") = "Tobalt"
        rw12("Nachname") = "Tonne"
        rw12("Geburtstag") = Convert.ToDateTime("5.8.1984")
        dt1.Rows.Add(rw11)
        dt1.Rows.Add(rw12)

        ' Tabelle Bestellungen:
        Dim dt2 As New DataTable("Bestellungen")
        Dim col2 As DataColumn = dt2.Columns.Add("Nr", Type.GetType("System.Int32"))
        col2.AllowDBNull = False
        col2.Unique = True
        col2.AutoIncrement = True
        col2.AutoIncrementStep = 1
        dt2.Columns.Add("Datum", GetType(System.DateTime))
        dt2.Columns.Add("Betrag", GetType(System.Decimal))
        dt2.Columns.Add("PersNr", GetType(System.Int32))           ' Fremdschlssel
        dt2.Columns.Add("Bemerkung", GetType(System.String))

        ' Zwei Datenstze hinzufgen:
        Dim rw21 As DataRow = dt2.NewRow()
        rw21("Datum") = Convert.ToDateTime("20.2.2006")
        rw21("Betrag") = Convert.ToDecimal("256,50")
        rw21("PersNr") = 0
        rw21("Bemerkung") = "per Nachname"
        dt2.Rows.Add(rw21)
        Dim rw22 As DataRow = dt2.NewRow()
        rw22("Datum") = Convert.ToDateTime("8.3.2006")
        rw22("Betrag") = Convert.ToDecimal("12,95")
        rw22("PersNr") = 0
        rw22("Bemerkung") = "per Scheck"
        dt2.Rows.Add(rw22)

        ' DataSet zusammenbauen (mit 1 : n Relation zwischen Kunden und Bestellungen):
        Dim ds As New DataSet()
        ds.Tables.Add(dt1)
        ds.Tables.Add(dt2)
        ds.Relations.Add("Person_Bestellungen", ds.Tables("Personen").Columns("Nr"), _
                                              ds.Tables("Bestellungen").Columns("PersNr"))
        Return ds
    End Function
End Class
