Imports System.Data.SqlClient
Imports System.Security.Permissions
Imports System.ComponentModel

Public Class Form1
    Private Const SQLSTR As String = "SELECT CustomerID, CompanyName, ContactName, ContactTitle FROM dbo.Customers ORDER BY CompanyName"
    Private csb As SqlConnectionStringBuilder = Nothing
    Private conn As SqlConnection = Nothing
    Private cmd As SqlCommand = Nothing
    Private ds As DataSet = Nothing

    Private count As Integer = 0
    Private Const msg As String = "{0} nderungen sind aufgetreten."


    ' Um SqlDependency nutzen zu knnen, muss Anwendung ber SqlClientPermission verfgen:
    Private Function canRequestNotifications() As Boolean
        Try
            Dim perm As New SqlClientPermission(PermissionState.Unrestricted)
            perm.Demand()
            Return True
        Catch
            Return False
        End Try
    End Function

    Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
        ' Clientberechtigung prfen:
        Button1.Enabled = canRequestNotifications()
        ' ConnectionString zusammenbauen:
        csb = New SqlConnectionStringBuilder()
        csb.DataSource = ".\SQLEXPRESS"
        csb.IntegratedSecurity = True
        csb.InitialCatalog = "Northwind"

        MyBase.OnLoad(e)
    End Sub


    Private Sub getData()
        ' Kehraus:
        ds.Clear()

        ' Sicherstellen, dass Command-Objekt nicht bereits mit einem Notification-Objekt verknpft ist:
        cmd.Notification = Nothing

        ' SqlDependency-Objekt erzeugen und mit Command-Objekt verbinden:
        Dim dep As New SqlDependency(cmd)
        ' Eventhandler anmelden:
        AddHandler dep.OnChange, AddressOf Me.dep_OnChange

        ' Daten laden und anzeigen:
        Dim da As SqlDataAdapter = New SqlDataAdapter(cmd)
        da.Fill(ds, "Customers")
        DataGridView1.DataSource = ds
        DataGridView1.DataMember = "Customers"
    End Sub

    ' mit Sql Server verbinden:
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        count = 0
        ToolStripStatusLabel1.Text = String.Format(msg, count)

        ' existierende Dependency-Connection anhalten und neu starten:
        SqlDependency.Stop(csb.ConnectionString)
        SqlDependency.Start(csb.ConnectionString)
        If IsNothing(conn) Then conn = New SqlConnection(csb.ConnectionString)
        If IsNothing(cmd) Then cmd = New SqlCommand(SQLSTR, conn)
        If IsNothing(ds) Then ds = New DataSet()
        getData()
    End Sub

    ' Der OnChange-Eventhandler: 
    Private Sub dep_OnChange(ByVal sender As Object, ByVal e As SqlNotificationEventArgs)

        ' Da das OnChange-Event nicht im Thread der Benutzerschnittstelle (UI-Thread),
        ' sondern im Arbeitsthread (aus dem Threadpool) auftritt, ist ein Aktualisieren
        ' der Benutzerschnittstelle zunchst nicht mglich. 
        ' Der folgende Code fhrt deshalb die notwendigen Aktionen aus, 
        ' um vom Arbeitstread  zum UI-Thread umzuschalten und die Ereignisbehandlung erneut zu registrieren.

        Dim isi As ISynchronizeInvoke = CType(Me, ISynchronizeInvoke)

        ' Wenn InvokeRequired true liefert, wird Code noch im Arbeitsthread ausgefhrt:
        If isi.InvokeRequired Then
            ' Delegate fr Thread-Schalter erzeugen:
            Dim tmpDeleg As New OnChangeEventHandler(AddressOf Me.dep_OnChange)
            Dim args() As Object = {sender, e}
            ' Daten-Marshalling vom Arbeitsthread zum UI-Thread:
            isi.BeginInvoke(tmpDeleg, args)
            Return
        End If

        ' ab hier wird Code wieder im UI-Thread ausgefhrt:

        ' Notification-Handler wird nicht mehr bentigt:
        Dim dep As SqlDependency = CType(sender, SqlDependency)
        RemoveHandler dep.OnChange, AddressOf Me.dep_OnChange

        count += 1
        ToolStripStatusLabel1.Text = String.Format(msg, count)

        ' Zusatzinfos anzeigen:
        ListBox1.Items.Clear()
        ListBox1.Items.Add("Type: " + e.Type.ToString())
        ListBox1.Items.Add("Source: " + e.Source.ToString())
        ListBox1.Items.Add("Info: " + e.Info.ToString())

        ' genderte Daten laden und anzeigen:
        getData()
    End Sub

    ' Der Abgesang:
    Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
        SqlDependency.Stop(csb.ConnectionString)
        If Not IsNothing(conn) Then conn.Close()
        MyBase.OnClosed(e)
    End Sub
End Class
